"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.KoconutPair = exports.Pair = void 0;

var _get2 = _interopRequireDefault(require("@babel/runtime/helpers/get"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _module = require("../../../module.internal");

var _KoconutBoolean = require("./KoconutBoolean");

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = (0, _getPrototypeOf2["default"])(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = (0, _getPrototypeOf2["default"])(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return (0, _possibleConstructorReturn2["default"])(this, result); }; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

/**
 * Represents a generic pair of two Values. There is no meaning attached to values
 * in this class. It can be used for any purpose. Pair exhibits values semantics,
 * i.e. two pairs are equal if both components are equal. However, if any of those
 * two values are instance of class, you'd better make the class explicitly distinguishable
 * by inheriting {@link KoconutEquatable}. Please, check the {@link Pair.equalsTo example of 'equalsTo' method}
 * @see
 * <pre>
 * -- Base --
 * {@link KoconutPair}, {@link Entry}, {@link KoconutEntry}
 * 
 * -- Protocol --
 * {@link KoconutEquatable}
 * </pre>
 * 
 * @param FirstType The type of the first value.
 * @param SecondType The type of the second value.
 */
var Pair = /*#__PURE__*/function () {
  (0, _createClass2["default"])(Pair, null, [{
    key: "from",

    /**
     * Create a {@link Pair} instance by iterable two values pair.
     * 
     * @param pair Values pair of first/second as iterable.
     * ```typescript
     * const myPair = Pair.from(["Apex","Captain"])
     * console.log(myPair)
     * // ↑ Pair { firstElement: 'Apex', secondElement: 'Captain' }
     * ```
     */
    value: function from(pair) {
      return new Pair(pair[0], pair[1]);
    }
    /**
     * Constructor of {@link Pair}.
     * 
     * @param firstElement FirstType element.
     * @param secondElement SecondType element.
     */

  }]);

  function Pair(firstElement, secondElement) {
    (0, _classCallCheck2["default"])(this, Pair);
    (0, _defineProperty2["default"])(this, "firstElement", void 0);
    (0, _defineProperty2["default"])(this, "secondElement", void 0);
    this.firstElement = firstElement;
    this.secondElement = secondElement;
  }
  /**
   * Returns the first value of this first/second pair.
   * @returns The first value of this first/second pair.
   */


  (0, _createClass2["default"])(Pair, [{
    key: "toString",

    /**
     * Turns this {@link Pair} instance into a simple JSON object string.
     * 
     * @example
     * ```typescript
     * const myPair = Pair.from(["Apex","Captain"])
     * console.log(myPair.toString()) // Or, you can use console.log(`${myPair}`)
     * // ↑ {"first":"Apex","second":"Captain"}
     * ```
     */
    value: function toString() {
      return JSON.stringify({
        first: this.first,
        second: this.second
      });
    }
    /**
     * Turns this {@link Pair} instance into a simple array.
     * 
     * @example
     * ```typescript
     * const myPair = Pair.from(["Apex","Captain"])
     * console.log(myPair.toArray())
     * // ↑ [ 'Apex', 'Captain' ]
     * ```
     */

  }, {
    key: "toArray",
    value: function toArray() {
      return [this.first, this.second];
    }
    /**
     * Turns this {@link Pair} instance into a simple {@link Entry}
     * @example
     * ```typescript
     * const myPair = Pair.from(["Apex","Captain"])
     * console.log(myPair.toEntry())
     * // ↑ Entry { keyElement: 'Apex', valueElement: 'Captain' }
     * ```
     */

  }, {
    key: "toEntry",
    value: function toEntry() {
      return new _module.Entry(this.first, this.second);
    }
    /**
     * Class {@link Pair} implments {@link KoconutEquatable}. The '{@link KoconutEquatable.equalsTo equalsTo}' method of
     * this is basically check each individual element (first/second) are same or not. When the type of each element
     * is child of {@link KoconutEquatable}, it'd be done by using its '{@link KoconutEquatable.equalsTo equalsTo}' method.
     * Otherwise, it'd be done simply by '==' operator.
     * @param other 
     * 
     * @example
     * ```typescript
     *   // Case 1 -- All values are simply number or string 
     *   const myPairCase1_01 = Pair.from([10, 20])
     *   const myPairCase1_02 = Pair.from([10, 20])
     *   console.log(`${myPairCase1_01.equalsTo(myPairCase1_02)}`)
     *   // ↑ true
     *
     *   const myPairCase1_03 = Pair.from(["Apex", "Captain"])
     *   const myPairCase1_04 = Pair.from(["Apex", "Captain"])
     *   console.log(`${myPairCase1_03 == myPairCase1_04}`)
     *   // ↑ false
     *   console.log(`${myPairCase1_03.equalsTo(myPairCase1_04)}`)
     *   // ↑ true
     *
     *   // Case 2 -- First Type is indistinguishable class
     *   class MyClass {
     *       private value : string
     *       constructor(value : string) {
     *           this.value = value
     *       }
     *   }
     *   const myPairCase2_01 = Pair.from([new MyClass("Apex"), "Captain"])
     *   const myPairCase2_02 = Pair.from([new MyClass("Apex"), "Captain"])
     *   console.log(`${myPairCase2_01.equalsTo(myPairCase2_02)}`)
     *   // ↑ false
     *
     *   // Case 3 -- First Type is distinguishable class 
     *   class MyDistinguishableClass implements KoconutEquatable {
     *       private value : string
     *       constructor(value : string) {
     *           this.value = value
     *       }
     *       equalsTo(other : MyDistinguishableClass) : boolean {
     *           return this.value == other.value
     *       }
     *   }
     *   const myPairCase3_01 = Pair.from([new MyDistinguishableClass("Apex"), "Captain"])
     *   const myPairCase3_02 = Pair.from([new MyDistinguishableClass("Apex"), "Captain"])
     *   console.log(`${myPairCase3_01.equalsTo(myPairCase3_02)}`)
     *   // ↑ true
     * ```
     */

  }, {
    key: "equalsTo",
    value: function equalsTo(other) {
      var _this = this;

      if (_module.KoconutTypeChecker.checkIsEquatable(this.firstElement) && _module.KoconutTypeChecker.checkIsEquatable(other.firstElement)) {
        var firstCompareResult = this.firstElement.equalsTo(other.firstElement);

        if (firstCompareResult instanceof _module.KoconutPrimitive) {
          var koconutToReturn = new _KoconutBoolean.KoconutBoolean();
          koconutToReturn.setProcessor( /*#__PURE__*/(0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee() {
            var thisValue, compareResult;
            return _regenerator["default"].wrap(function _callee$(_context) {
              while (1) {
                switch (_context.prev = _context.next) {
                  case 0:
                    thisValue = koconutToReturn['data'];

                    if (thisValue) {
                      _context.next = 3;
                      break;
                    }

                    return _context.abrupt("return", false);

                  case 3:
                    if (!(_module.KoconutTypeChecker.checkIsEquatable(_this.secondElement) && _module.KoconutTypeChecker.checkIsEquatable(other.secondElement))) {
                      _context.next = 14;
                      break;
                    }

                    compareResult = _this.secondElement.equalsTo(other.secondElement);

                    if (!(compareResult instanceof _module.KoconutPrimitive)) {
                      _context.next = 11;
                      break;
                    }

                    _context.next = 8;
                    return compareResult["yield"]();

                  case 8:
                    return _context.abrupt("return", _context.sent);

                  case 11:
                    return _context.abrupt("return", compareResult);

                  case 12:
                    _context.next = 15;
                    break;

                  case 14:
                    return _context.abrupt("return", _this.secondElement == other.secondElement);

                  case 15:
                  case "end":
                    return _context.stop();
                }
              }
            }, _callee);
          })));
          return koconutToReturn;
        } else {
          if (!firstCompareResult) return false;

          if (_module.KoconutTypeChecker.checkIsEquatable(this.secondElement) && _module.KoconutTypeChecker.checkIsEquatable(other.secondElement)) {
            var secondCompareResult = this.secondElement.equalsTo(other.secondElement);
            if (secondCompareResult instanceof _module.KoconutPrimitive) return _KoconutBoolean.KoconutBoolean['fromPrimitive'](secondCompareResult);else return secondCompareResult;
          } else return this.secondElement == other.secondElement;
        }
      } else {
        if (this.firstElement != other.firstElement) return false;

        if (_module.KoconutTypeChecker.checkIsEquatable(this.secondElement) && _module.KoconutTypeChecker.checkIsEquatable(other.secondElement)) {
          var compareResult = this.secondElement.equalsTo(other.secondElement);
          if (compareResult instanceof _module.KoconutPrimitive) return _KoconutBoolean.KoconutBoolean['fromPrimitive'](compareResult);else return compareResult;
        } else return this.secondElement == other.secondElement;
      }
    }
  }, {
    key: "first",
    get: function get() {
      return this.firstElement;
    }
    /**
     * Returns the second value of this first/second pair.
     * @returns The second value of this first/second pair.
     */

  }, {
    key: "second",
    get: function get() {
      return this.secondElement;
    }
  }]);
  return Pair;
}();
/**
 * Koconut Wrapper class for {@link Pair}
 * 
 * @see
 * <pre>
 * -- Base --
 * {@link Pair}, {@link Entry}, {@link KoconutPair}
 * 
 * -- Protocol --
 * {@link KoconutEquatable}
 * </pre>
 * 
 * @param FirstType Check for {@link Pair}
 * @param SecondType Check for {@link Pair}
 */


exports.Pair = Pair;

var KoconutPair = /*#__PURE__*/function (_KoconutPrimitive) {
  (0, _inherits2["default"])(KoconutPair, _KoconutPrimitive);

  var _super = _createSuper(KoconutPair);

  /**
   * Constructor of {@link KoconutPair}
   * 
   * @param first FirstType element of inner {@link Pair} instance.
   * @param second SecondType element of inner {@link Pair} instance.
   */
  function KoconutPair() {
    var _this2;

    var first = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
    var second = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    (0, _classCallCheck2["default"])(this, KoconutPair);
    if (first != null && second != null) _this2 = _super.call(this, new Pair(first, second));else _this2 = _super.call(this);
    return (0, _possibleConstructorReturn2["default"])(_this2);
  }
  /**
   * Processes all the chained object and returns original {@link KoconutPair} instance.
   * 
   * @category Processor
   * 
   * @since 1.0.15
   * 
   * @example
   * ```typescript
   * const koconutPair = await new KoconutPair(0, 1)
   *                                     .retrieve()
   *                                     
   * console.log(koconutPair)
   * // ↑ KoconutPair {
   * //   isValidated: true,
   * //   data: Pair { firstElement: 0, secondElement: 1 }
   * // }
   * ```
   */


  (0, _createClass2["default"])(KoconutPair, [{
    key: "retrieve",
    value: function () {
      var _retrieve = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2() {
        return _regenerator["default"].wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return (0, _get2["default"])((0, _getPrototypeOf2["default"])(KoconutPair.prototype), "retrieve", this).call(this);

              case 2:
                return _context2.abrupt("return", this);

              case 3:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function retrieve() {
        return _retrieve.apply(this, arguments);
      }

      return retrieve;
    }()
    /**
     * Class {@link KoconutPair} implements {@link KoconutEquatable}. The equality check process
     * is done by using '{@link Pair.equalsTo equalsTo method of Pair}'
     * @param other 
     */

  }, {
    key: "equalsTo",
    value: function equalsTo(other) {
      if (this.data != null && other.data != null) return this.data.equalsTo(other.data);
      return false;
    }
  }]);
  return KoconutPair;
}(_module.KoconutPrimitive);

exports.KoconutPair = KoconutPair;